/*	Facebook Un-Appifier 
 *
 *	The content script (unlink.js) actually handles the meat of removing the app wall. 
 *	This script handles the UI side, showing a small icon in the address bar that allows the user to turn on or off this functionality.
 *
 *	Created by Chris Lim (pocketlim.com) - Original experiment at pocketlim.com/unappify
 *
 *	Special thanks to:
 *		Jason Yee - for pointing me the way toward making a Chrome extension
 *		Hyung Lee - for giving me some UI advice
 *		Azat - for unknowingly letting me use his FB profile to test (he reads a lot of app walled stuff... tee hee)
 *		Chris Alvino - for wanting to actually use this, woo!
 */

// This debug is handled from the unlink.js side. Leave this false.
// To turn on debug, switch to unlink.js and turn it on there.
var debug = false;
function log(msg) {
	if ( debug && console && console.log ) {
		console.log(msg);
	}
}

// Chooses which icon, disabled or enabled
function selectIcon(tab, action) {
	if ( action == 'on' ) {
		chrome.pageAction.setIcon({path: 'fb-unapp-19.png', tabId: tab.id});
	} else if ( action == 'off' ) {
		chrome.pageAction.setIcon({path: 'fb-unapp-19-dis.png', tabId: tab.id});
	}
}

// Handles messages from the content script
function processMessage(request, sender, sendResponse) {
	var action = request.action;
	debug = request.debug;
	
	chrome.tabs.getSelected(null, function(tab) {
		log('request from js: ' + action);
		selectIcon(tab, action);
	});
}

// enables the unappify icon if we match the right page (facebook.com)
function checkForValidUrl(tabId, changeInfo, tab) {
	var search = tab.url.indexOf('facebook.com');
	if ( search > -1 && search < 30 ) {
		chrome.pageAction.show(tabId);
		chrome.tabs.sendRequest(tab.id, {action: 'status'}, function(response) {
			log(response);
		});
	}
}

// listen for messages from other parts of the extension
chrome.extension.onRequest.addListener(processMessage);

// listen for any changes to the URL of any tab.
chrome.tabs.onUpdated.addListener(checkForValidUrl);

(function($) {
	var menu = $('.menu');
	menu.on( 'click', 'li', function(e) {
		// figure out what user clicked on
		var action = $(e.target).closest('li').attr('id');

		chrome.tabs.getSelected(null, function(tab) {
			selectIcon(tab, action);

			// tell the content script what the user chose
			chrome.tabs.sendRequest(tab.id, {action: action}, function(response) { 
				//log(response); 
			});

			// updating the tab closes the popup
			chrome.tabs.update(tab.id, { selected: true } );
		});
	});
})(jQuery);