/*	Facebook Un-Appifier 
 *
 *	Removes authorization links to random Facebook apps that try to access and post on your behalf. 
 *		Because privacy stealing apps suck. And those authorization links are annoying.
 *
 *	Created by Chris Lim (pocketlim.com) - Original experiment at pocketlim.com/unappify
 *
 *	Special thanks to:
 *		Jason Yee - for pointing me the way toward making a Chrome extension
 *		Hyung Lee - for giving me some UI advice
 *		Azat - for unknowingly letting me use his FB profile to test (he reads a lot of app walled stuff... tee hee)
 *		Chris Alvino - for wanting to actually use this, woo!
 */

// this global debug controls both background extension and this content script
var debug = false;

// log function depends on debug
function log(msg) {
	if ( debug && console && console.log ) {
		console.log(msg);
	}
}

// tell the extension what the status is
function sendStatus(action) {
	chrome.extension.sendRequest({action: action, debug: debug}, function(response) { 
		//log(response); 
	});
}

// initializes our localStorage value
function init() {
	// if not used before, set to true (default on)
	var status = localStorage.getItem('fb_unappify_status');
	if ( status == null ) {
		localStorage.setItem('fb_unappify_status', true);
	}

	var action = localStorage.getItem('fb_unappify_status') == 'true' ? 'on' : 'off';
	sendStatus(action);
	log('init: ' + action);
}

// initialize
init();

(function($) {
	var gTime = 0;
	// performance
	var runTimer = 0,
		hasRunYet = false;

	// cache items
	var $body = $('body'),
		// exceptions don't work using the facebook link redirector, we will just directly link them
		exceptions = ['socialcam.com'],
		offset = 'redirect_uri='.length,
		// facebook's link redirector, will warn user that they're heading to an external site unless they come with hashes below
		fbLinker = 'http://www.facebook.com/l.php?u=',
		// these hashes prevent facebook from warning you that you're heading to an external site - this is also annoying
		// very hacky, I just took it from some random link and it seems to work for EVERY OTHER LINK
		fbLinkerHashes = '?&pf_rd_s=center-2&pf_rd_r=12K3YN7SS12JKJX315CD&pf_rd_t=101&pf_rd_p=280598007&pf_rd_i=468294&h=WAQG88SAkAQHEphM-SRBg-pVVToPAHoXn_09KGGjJLN3XiA&enc=AZNF-sLztyXoWQw6d6XrZKam3dxpQyx5w72tl4h40oo8rkdExzqqPtjhHLG1ETJpoYr8wbOcHA6l6q8Qo8mm6-1S'; //'?pf_rd_m=A3P5ROKL5A1OLE&pf_rd_s=center-2&pf_rd_r=12K3YN7SS12JKJX315CD&pf_rd_t=101&pf_rd_p=280598007&pf_rd_i=468294&h=WAQG88SAkAQHEphM-SRBg-pVVToPAHoXn_09KGGjJLN3XiA&enc=AZNF-sLztyXoWQw6d6XrZKam3dxpQyx5w72tl4h40oo8rkdExzqqPtjhHLG1ETJpoYr8wbOcHA6l6q8Qo8mm6-1S';
	
	$body.on('click', 'a[data-unapp]', function(e) {
		e.preventDefault(); 
		e.stopPropagation();
		var href = $(e.target).closest('a[data-unapp]').attr('href');
		log('opening new window: ' + href);
		window.open(href, '_blank');
	});

	function run(e) {
		var status = localStorage.getItem('fb_unappify_status');
		if ( status == 'true' ) {
			// run a half second after the DOM finishes uploading
			clearTimeout(runTimer);
			runTimer = setTimeout( function() {
				hasRunYet = false;
				var sTime = +new Date;
				runUnappify(e);
				gTime += (+new Date - sTime);

				// performance monitoring
				// log('normal fired\ntotal time: ' + gTime + ' ms');
			}, 500);

			// separate backup timer 
			if ( !hasRunYet ) {
				hasRunYet = true;

				// if within a half second after the first DOM update, we haven't run yet
				// run it now
				setTimeout( function() {
					if ( !hasRunYet ) return;

					hasRunYet = false;
					var sTime = +new Date;
					runUnappify(e);
					gTime += (+new Date - sTime);

					// performance monitoring
					// log('backup fired\ntotal time: ' + gTime + ' ms');
				}, 500);
			}
		}
	}

	// restores links to their original app requesting version
	function reAppify(e) {
		// match links that we've worked on previously
		var matchedA = $('a[data-unapp]');
		if ( !matchedA.length ) return;

		matchedA.each( function(i, el) {
			var $el = $(el).removeAttr('data-unapp'),
				rel = $el.attr('data-rel'),
				href = $el.attr('data-orig'),
				onmousedown = $el.attr('data-onmousedown');
			
			// restore optional stuff
			if ( rel ) { $el.attr('rel', rel).removeAttr('data-rel'); }
			if ( onmousedown ) { $el.attr('onmousedown', onmousedown).removeAttr('data-onmousedown'); }
			
			// restore href
			$el.attr('href', $el.attr('data-orig')).removeAttr('data-orig');
		});
	}

	// removes app request and allows user to just see content
	function runUnappify(e) {
		// match only links that have app walls, that we did not touch already
		var matchedA = $('a[href*="redirect_uri"]').not('[data-unapp]')
						.add( $('a[onmousedown]').not('[data-unapp]') );

		// if no matches, short circuit
		if ( !matchedA.length ) return;

		matchedA.each( processLink );
	}

	function processLink(i,el) {
		var $el = $(el).attr('data-unapp','1'), // ensure that we don't try to parse this url again
			hasImg = $el.find('img').length,
			href = $el.attr('href'),
			indx = href.indexOf('redirect_uri=');

		$el.attr('data-onmousedown', $el.attr('onmousedown')).removeAttr('onmousedown');

		// if required string is not there, short circuit
		if ( indx == -1 ) return;

		// parse out the URL - Todo: I should probably really use REGEXes
		var decoded = decodeURIComponent(href.substr(indx + offset)),
			isException = false;

		for ( var i in exceptions ) {
			i = exceptions[i];
			if ( decoded.indexOf(i) != -1 ) {
				isException = true;
			}
		}

		var hasQues = decoded.indexOf('?') != -1,
			hasAmp = decoded.indexOf('&response_type') != -1,
			endIdx = (hasQues) ? decoded.indexOf('?') : (hasAmp) ? decoded.indexOf('&') : undefined,
			parsedLink = decoded.substr( 0, endIdx ),
			// create facebook redirect URL, with the hash to skip the warning
			finalLink = parsedLink; // (isException) ? parsedLink : fbLinker + parsedLink + fbLinkerHashes;

		// replace the original URL and ensure to open in a new window; prevent opening a facebook dialog (remove rel attr)
		$el.attr('href', finalLink).attr('target','_blank').attr('data-orig',href).attr('data-rel', $el.attr('rel')).removeAttr('rel');

		log('Original: ' + href + '\nLinked: ' + finalLink + '\n----------');
	}

	// when more profile/data/whatever appears in the DOM, ensure to parse it for links
	$body.on( "DOMSubtreeModified", run );

	// processes the message from other parts of the extension
	function processMessage(request, sender, sendResponse) {
		log(request.action);
		if (request.action == 'on') {
			localStorage.setItem('fb_unappify_status', true);
			runUnappify();
		} else if (request.action == 'off') {
			localStorage.setItem('fb_unappify_status', false);
			reAppify();
		} else if (request.action == 'status') {
			init();
		}

		// tell the extension what the action is 
		sendStatus(request.action);
	}

	// listen for messages from other parts of the extension
	chrome.extension.onRequest.addListener( processMessage );
})(jQuery);